/*
   Copyright (C) 2001 Thomas Capricelli <orzel@freehackers.org>
*/

#include <config.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

/* std c */
#include <stdlib.h>

/* Qt */
#include <qmessagebox.h>
#include <qlist.h>
#include <qdatastream.h>
#include <qdatetime.h>
#include <qregexp.h>
#include <qfile.h>

/* KDE */
#include <kdebug.h>
#include <kmessagebox.h>

/* Koffice */
#include <koFilterChain.h>
#include <kgenericfactory.h>

/* Opale */
#include <cashimport.h>
#include "old_cash.h"

#define INT_SWAP(a) (\
        (((a) & 0xff000000U) >> 24) | \
        (((a) & 0x000000ffU) << 24) | \
        (((a) & 0x0000ff00U) <<  8) | \
        (((a) & 0x00ff0000U) >>  8))

#define USE_SWAP(_x)	(((unsigned int)_x)=INT_SWAP(((unsigned int)_x)))


/*
 * factory stuff
 */
typedef KGenericFactory<CASHImport, KoFilter> CashImportFactory;
K_EXPORT_COMPONENT_FACTORY( libcashimport, CashImportFactory( "cashimport" ) );


CASHImport::CASHImport(KoFilter *, const char *, const QStringList&)
	:KoFilter()
{
}

KoFilter::ConversionStatus CASHImport::convert( const QCString& from, const QCString& to )
{
	QString fileIn( m_chain->inputFile() );
	bool need_swap = false;

	kdDebug(55000) << "CASHImport::filter : entering " << endl;

	// mimetype check
	if(to!="application/x-opale" || (from!="application/x-cash-le" && from!="application/x-cash-be")) {
		kdWarning(55000) << "Invalid mimetypes " << from << " " << to << endl;
		return KoFilter::NotImplemented;
	}

#if defined(WORDS_BIGENDIAN)
	if ("application/x-cash-le" == from)
#else
	if ("application/x-cash-be" == from)
#endif
		need_swap = true;

	QFile in(fileIn);
	if(!in.open(IO_ReadOnly)) {
		kdDebug(55000) << "Unable to open input file!" << endl;
		KMessageBox::sorry( 0L, i18n("Cash filter can't open input file - please report.") );
		in.close();
		return KoFilter::FileNotFound;
	}


	// header 
	QString str;

	str += "<?xml version=\"1.0\" encoding=\"UTF-8\"?> <!DOCTYPE opale >\n";
	str += "<account mime=\"application/x-opale\" email=\"orzel@freehackers.org\" author=\"Thomas Capricelli\" account_name=\"still_fake bank name\" editor=\"Opale\" >\n\n";

/*
from DTD :
	<howlist>
		<how name="Check" num="0" />
		<how name="asdsf" num="1" />
		<how name="asdf" num="2" />
	</howlist>

cash : 
	Style : (0-Check  1-VISA  2-TIP  3-Credit Transfer  4-Retrait 5-Other)
*/
	str += "\t<howlist>\n";
	str += "\t\t<how name=\"Check\" num=\"0\" />\n";
	str += "\t\t<how name=\"Visa\" num=\"1\" />\n";
	str += "\t\t<how name=\"TIP\" num=\"2\" />\n";
	str += "\t\t<how name=\"Credit Transfer\" num=\"3\" />\n";
	str += "\t\t<how name=\"Retrait\" num=\"4\" />\n";
	str += "\t\t<how name=\"Other\" num=\"5\" />\n";
	str += "\t</howlist>\n\n";

	str += "\t<categorylist>\n";
	str += "\t\t<category name=\"Unknown\" num=\"0\" />\n";
	str += "\t</categorylist>\n\n";

	str += "\t<patternlist>\n";
	str += "\t</patternlist>\n\n";
	// magic ? 
	QDataStream stream(&in);

	char magic[]="CASH_MAGICxxx";
	stream >> (unsigned char)magic[0]; stream >> (unsigned char)magic[1];
	stream >> (unsigned char)magic[2]; stream >> (unsigned char)magic[3];
	stream >> (unsigned char)magic[4]; stream >> (unsigned char)magic[5];
	stream >> (unsigned char)magic[6]; stream >> (unsigned char)magic[7];
	stream >> (unsigned char)magic[8]; stream >> (unsigned char)magic[9];
	stream >> (unsigned char)magic[10];

	if ( magic[10]!=0 ||
		magic[9]!='C' || magic[8]!='I' ||
		magic[7]!='G' || magic[6]!='A' ||
		magic[5]!='M' || magic[4]!='_' ||
		magic[3]!='H' || magic[2]!='S' ||
		magic[1]!='A' || magic[0]!='C'  ) {

		QMessageBox::critical (0L, "Cash header problem",
	                              QString ("The Cash header is not correct. "
	                                       "Maybe it's not an cash file! <BR>"),
				    "Okay");
		return KoFilter::BadConversionGraph;
	}


	kdDebug(55000) << "HEADER OK ****************" << endl;

	emit sigProgress(2);

	// read the number of record
	stream >> (unsigned char)magic[0]; stream >> (unsigned char)magic[1];
	stream >> (unsigned char)magic[2]; stream >> (unsigned char)magic[3];
	stream >> (unsigned char)magic[4]; //stream >> (unsigned char)magic[5];
	magic[5] = 0;

	int n = atoi(magic);
	str += QString("\t<recordlist currency=\"FF\" >\n");
	for (int i=0; i<n; i++) {

		union {
			s_ope	ope;
			char	c[1];
		} ope;
		QString	name;

		// get the data
		for (unsigned int li=0; li< sizeof(s_ope); li++)
			stream >> (unsigned char) ope.c[li];

		if (need_swap) {
			USE_SWAP(ope.ope.how);
			USE_SWAP(ope.ope.when);
			USE_SWAP(ope.ope.much);
			USE_SWAP(ope.ope.flag);
		}

		// <record what="hop" amount="123" how="&lt;other>" when="987021934" />
		name = ope.ope.what;
		name.replace(QRegExp("\""), "'");
		name.replace(QRegExp("&"), "&amp;");
//		if (i>510 && i <517) printf("line %d : %s\n", i, (const char *)name.utf8() );

		str += QString(
			"\t\t<record what=\"%1\" amount=\"%2\" how=\"%3\" category=\"%3\" when=\"%4\" checked=\"%5\" />\n"
			).arg(name).arg(ope.ope.much).arg( ((ope.ope.how<6)?ope.ope.how:-1) ).arg(0).arg(ope.ope.when).arg((ope.ope.flag&OPE_FLAG_CHK)?"yes":"no");
			
		emit sigProgress(i*100/n);
	}
	str += "\t</recordlist>\n";



	emit sigProgress(100);

	// footer
	str += "</account>\n";

	// conclusion 
#if 0
	printf ("Text %s\n", (const char *) str.utf8());
#endif

	KoStoreDevice* out=m_chain->storageFile( "root", KoStore::Write );

	if (!out) {
	kdError(55000) << "Unable to open output file!" << endl;
	in.close  ();
	return KoFilter::StorageCreationError;
	}

	QCString cstring = str.utf8();
	out->writeBlock ( cstring, cstring.length() );

	in.close();
	return KoFilter::OK;
}

