/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

/* Qt */
#include <qpainter.h>
#include <qcombobox.h>
#include <qlineedit.h>

/* KDE */
#include <kdebug.h>
#include <klistview.h>
#include <klocale.h>

/* KOffice */
#include <koTemplateChooseDia.h> 
#include <KDChartTable.h>

/* Opale */
#include "opale_factory.h"
#include "opale_part.h"
#include "opale_view.h"
#include "input_fileoptions.h"


OpalePart::OpalePart( QWidget *parentWidget, const char *widgetName, QObject* parent, const char* name, bool singleViewMode )
    : KoDocument( parentWidget, widgetName, parent, name, singleViewMode )
{
}

bool OpalePart::initDoc()
{
	kdDebug(OPALE_DEBUG_STREAM) << "initDoc called" << endl;

	QString f;
	KoTemplateChooseDia::ReturnType ret;
	KoTemplateChooseDia::DialogType dlgtype;
 

	if (initDocFlags() != KoDocument::InitDocFileNew )
		dlgtype = KoTemplateChooseDia::Everything;
	else
		dlgtype = KoTemplateChooseDia::OnlyTemplates;

	ret = KoTemplateChooseDia::choose(  OpaleFactory::global(), f, "application/x-opale",
                                        "*.opale", "Opale",
					dlgtype, "opale_template");

	if ( ret == KoTemplateChooseDia::File ) {
		KURL url(f);
		return openURL( url );
	}

	if ( ret == KoTemplateChooseDia::Empty ) {

		records.clear();
		sl_howlist.clear();
		sl_categorylist.clear();

		s_currency =  i18n("Euro");
		s_account_name =  i18n("Bank's name");

		editFileOptions();

		return true;
	}

/* TODO
    if ( ret == KoTemplateChooseDia::Template )
    {
        QFileInfo fileInfo( f );
        QString fileName( fileInfo.dirPath( true ) + "/" +
            fileInfo.baseName() + ".kst" );
        resetURL();
        loadNativeFormat( fileName );
        setEmpty();
        initConfig();
        return true;
    }
   */



    return false;
}


KoView* OpalePart::createViewInstance( QWidget* parent, const char* name )
{
	KoView *tmp = new OpaleView( this, parent, name );
	syncPatterns();
	return tmp;
}

bool OpalePart::loadXML( QIODevice *, const QDomDocument &doc )
{
	QDomNode n;
	int i=0;

	kdDebug(OPALE_DEBUG_STREAM) << "begin of loadXML" << endl;
	/* check document type */
	if ( doc.doctype().name() != "opale")
		return false;

	/* check mimetype */
	kdDebug(OPALE_DEBUG_STREAM) << "check mimetype" << endl;
	QDomElement account = doc.documentElement();
	if ( account.attribute( "mime" ) != "application/x-opale" )
		return false;

	/* read account information */
	s_currency = account.attribute("currency");
	if ("" == s_currency ) s_currency="Euros";
	s_account_name = account.attribute("account_name");

	/* read the how section */
	kdDebug(OPALE_DEBUG_STREAM) << "read the how section... " << endl;
	QDomElement howlist = account.namedItem("howlist").toElement();
	sl_howlist.clear();
	n = howlist.firstChild();
	i=0;
	while (!n.isNull()) {
		QDomElement e = n.toElement();
		if (i!=e.attribute("num").toInt()) {
			kdDebug(OPALE_DEBUG_STREAM) << "num=" << e.attribute("num").toInt()
				<< " in how=" << e.attribute("name").latin1() << " element,"
				<< i << " was expected, file is corrupted" << endl;
			return false;
		}
		i++;
		sl_howlist.append(e.attribute("name"));
		n = n.nextSibling();
	}
	kdDebug(OPALE_DEBUG_STREAM) << i << " elements" << endl;

	/* read the category section */
	kdDebug(OPALE_DEBUG_STREAM) << "read the category section... " << endl;
	QDomElement categorylist = account.namedItem("categorylist").toElement();
	sl_categorylist.clear();
	n = categorylist.firstChild();
	i=0;
	while (!n.isNull()) {
		QDomElement e = n.toElement();
		if (i!=e.attribute("num").toInt()) {
			kdDebug(OPALE_DEBUG_STREAM) << "num=" << e.attribute("num").toInt()
				<< " in categorys=" << e.attribute("name").latin1() << " element,"
				<< i << " was expected, file is corrupted" << endl;
			return false;
		}
		i++;
		sl_categorylist.append(e.attribute("name"));
		n = n.nextSibling();
	}
	kdDebug(OPALE_DEBUG_STREAM) << i << " elements" << endl;

	/* read the patterns section */
	kdDebug(OPALE_DEBUG_STREAM) << "read the patterns section..." << endl;
	QDomElement patternslist = account.namedItem("patternlist").toElement();

	patterns.clear();
	n = patternslist.firstChild();
	i = 0;
	while (!n.isNull()) {
		patterns.append( new OpalePattern(n.toElement()) );
		n = n.nextSibling();
		i++;
	}
	syncPatterns();
	kdDebug(OPALE_DEBUG_STREAM) << i << " elements" << endl;

	/* read the data section */
	kdDebug(OPALE_DEBUG_STREAM) << "read the data section..." << endl;
	QDomElement data = account.namedItem("recordlist").toElement();

	records.clear();
	n = data.firstChild();
	i=0;
	while (!n.isNull()) {
		records.append( new OpaleRecord(n.toElement()) );
		n = n.nextSibling();
		i++;
	}
	kdDebug(OPALE_DEBUG_STREAM) << i << " elements" << endl;

	kdDebug(OPALE_DEBUG_STREAM) << "end of loadXML" << endl;

	setModified(false);
	return true;
}


/*

<?xml version="1.0" encoding="UTF-8"?><!DOCTYPE opale >
<account
	mime="application/x-opale"
	editor="Opale"
	currency="Euro" 
	account_name="bank_name" >

	<howlist>
		<how name="Check" num="0" />
		<how name="asdsf" num="1" />
		<how name="asdf" num="2" />
	</howlist>

	<categorylist>
		<category name="Check" num="0" />
		<category name="asdsf" num="1" />
		<category name="asdf" num="2" />
	</categorylist>

	<data>
		<record amount="12300" when="12352351235" what="Burps pour la shbrob" how="3" category="1" />
		<record amount="12300" when="12352351235" what="Burps pour la shbrob" how="3" category="1" />
		<record amount="12300" when="12352351235" what="Burps pour la shbrob" how="3" category="1" />
	</data>
</account>


 */

QDomDocument OpalePart::saveXML()
{
	QDomDocument doc("opale");
	doc.appendChild( doc.createProcessingInstruction( "xml", "version=\"1.0\" encoding=\"UTF-8\"" ) );
	QDomElement account = doc.createElement( "account" ); 
	account.setAttribute( "mime",	"application/x-opale" );
	account.setAttribute( "editor",	"Opale" );
	account.setAttribute( "account_name", s_account_name );
	account.setAttribute( "currency", s_currency);
	doc.appendChild( account );

	/* record the how names */
	QDomElement howlist = doc.createElement( "howlist" ); 

	for ( uint i=0; i< sl_howlist.count(); i++) {
		QDomElement record = doc.createElement( "how" );
		record.setAttribute("name", sl_howlist[i]);
		record.setAttribute("num", i);
		howlist.appendChild(record);
        }
	account.appendChild(howlist);

	/* record the category names */
	QDomElement categorylist = doc.createElement( "categorylist" ); 

	for ( uint i=0; i< sl_categorylist.count(); i++) {
		QDomElement record = doc.createElement( "category" );
		record.setAttribute("name", sl_categorylist[i]);
		record.setAttribute("num", i);
		categorylist.appendChild(record);
        }
	account.appendChild(categorylist);

	/* record the patterns */
	QDomElement patternslist = doc.createElement( "patternlist" ); 
	for (OpalePattern *p = patterns.first(); p; p=patterns.next()) {
		patternslist.appendChild( p->domElement(doc));
	}
	account.appendChild(patternslist);


	/* record the data */
	QDomElement data = doc.createElement( "recordlist" ); 
	for (OpaleRecord *r = records.first(); r; r=records.next()) {
		data.appendChild( r->domElement(doc));
	}
	account.appendChild(data);

	setModified(false);
	return doc;
}


void OpalePart::fillDataGraph(KDChartTableData &chartData , QDateTime &begin, QDateTime &end, amount_t &a_min, amount_t &a_max)
{
	amount_t y, y_best, y_worst, y_best2,y_worst2;
	int i=0;
	OpaleRecord *r;

	QDateTime clip_begin = begin.addMonths(-1)
		, clip_end = end.addMonths(+1);

#if 0
	if (chartData.cols() < records.count() );
		chartData.expand(5, records.count() );
#else
	chartData.expand(5, records.count() );
#endif

	a_min = a_max = 0; 
	i=y=y_best=y_worst=y_best2=y_worst2=0;
	for (r = records.first(); r; r=records.next()) {
		// Those are the one displayed
		amount_t a = r->amount();
		y += a;

		if (r->qdt>clip_end) break;

		// compute
		if (a>=0) {
			y_best += a;
			y_best2 += a;
			if (r->checked() ) y_worst += a;
			if (r->checked() || r->isPast()  ) y_worst2 += a;
		} else {
			y_worst += a;
			y_worst2 += a;
			if (r->checked() ) y_best += a;
			if (r->checked() || r->isPast()  ) y_best2 += a;
		}
		// min/max
		if (begin < r->qdt &&  r->qdt <= end) {
			if (y_best>a_max) a_max = y_best;
			if (y_worst<a_min) a_min = y_worst;
		}

		//fill
		if (r->qdt>clip_begin) {
			chartData.setCell(0, i , KDChartData( (double)(y/100.),  r->qdt));
			chartData.setCell(1, i , KDChartData( (double)(y_best/100.),  r->qdt));
			chartData.setCell(2, i , KDChartData( (double)(y_worst/100.),  r->qdt));
			chartData.setCell(3, i , KDChartData( (double)(y_best2/100.),  r->qdt));
			chartData.setCell(4, i , KDChartData( (double)(y_worst2/100.),  r->qdt));
		}
		i++;
	}

	// safeguard
	if (a_max == a_min ) {
		a_min = -10 * 100;
		a_max = +10 * 100;
	}
}

QPtrDict<OpaleRecord> * OpalePart::fillList(Total *total, KListView *qlv)
{
	QPtrDict<OpaleRecord> *dict = new QPtrDict<OpaleRecord>;
	total->global = 0l;	total->utt = 0l;
	total->uttsafe = 0l;	total->checked = 0l;

	KListViewItem *item;

	qlv->clear();

	for (OpaleRecord *r = records.first(); r; r=records.next()) {
		item = r->fillListView(total, qlv, s_currency, sl_howlist, sl_categorylist);
		if (item) dict->insert(item, r);
	}

	return dict;
}


void OpalePart::fillCombos(QComboBox *how, QComboBox *category)
{
	how->insertStringList(sl_howlist);
	category->insertStringList(sl_categorylist);
}


int OpalePart::checkTypeCombos(QComboBox *type)
{
	unsigned int i;

//	kdDebug(OPALE_DEBUG_STREAM) << "checkTypeCombos : ok, item : " << type->currentItem()
//			<< ", count : " << sl_howlist.count() << ", text : "
//			<< type->currentText().latin1() << endl;

	for (i=0; i< sl_howlist.count(); i++)
		if (type->currentText() == sl_howlist[i]) {
			type->setCurrentItem(i);
			return i;
		}

	kdDebug(OPALE_DEBUG_STREAM) << "checkTypeCombos : adding" << endl;
	i = sl_howlist.count();
	sl_howlist.append(type->currentText());
	return i;
}


int OpalePart::checkCategoryCombos(QComboBox *category)
{
	unsigned int i;

//	kdDebug(OPALE_DEBUG_STREAM) << "checkCategoryCombos : ok, item : " << category->currentItem()
//			<< ", count : " << sl_categorylist.count() << ", text : "
//			<< category->currentText().latin1() << endl;

	for (i=0; i< sl_categorylist.count(); i++)
		if (category->currentText() == sl_categorylist[i]) {
			category->setCurrentItem(i);
			return i;
		}

	kdDebug(OPALE_DEBUG_STREAM) << "checkCategoryCombos : adding" << endl;
	i = sl_categorylist.count();
	sl_categorylist.append(category->currentText());
	return i;
}


void OpalePart::paintContent( QPainter& /*painter*/, const QRect& /*rect*/, bool /*transparent*/, double /*zoomX*/, double /* zoomY */ )
{
	/* needed by koDocument */ 
}


void OpalePart::deleteRecord(OpaleRecord *r)
{
	records.remove(r);
	touched();
}


void OpalePart::touched(void)
{
	setModified(true); emit dataUpdated();
}


void OpalePart::addRecord(OpaleRecord *newr)
{
	OpaleRecord *r;

	for ( r=records.first(); r != 0; r=records.next() )
		if (r->qdt > newr->qdt) {
			records.insert( records.at(), newr);
			touched();
			return;
		}
	records.append( newr);
	touched();
}


void OpalePart::addPattern(OpalePattern *newp)
{
	patterns.append( newp);
	syncPatterns();
	touched();
}


void OpalePart::syncPatterns(void)
{
	QStringList	namelist;
	for (unsigned int i=0; i< patterns.count(); i++)
		namelist.append( patterns.at(i)->name);
	emit newPatterns ( namelist );
}

int OpalePart::editFileOptions(QWidget *parent)
{
	
	int result;

	fileOptionsDialog dialog(parent, "edit_file_options", true);

	dialog.edit_currency->setText(currency());
	dialog.edit_account_name->setText(account_name());
	result = dialog.exec();

	if (result) {
		setInfos(dialog.edit_currency->text(), dialog.edit_account_name->text());
	}
	return result;
}




