/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

/* Qt */
#include <qpainter.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qspinbox.h>

/* KDE */
#include <kaction.h>
#include <kstdaction.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <klistview.h>
#include <kdebug.h>

/* Opale */
#include "opale_view.h"
#include "opale_factory.h"
#include "opale_part.h"
#include "opale_graph.h"
#include "input_record.h"
#include "input_pattern.h"
#include "input_repeat.h"



/*
 * OpaleView 
 */

#define PUT_TOTAL(NAME,var)								\
	var = new QLabel( NAME , this);							\
	var->setFont( _font );								\
	l->addWidget( var);								\
	var = new QLabel( " 0.00" , this);						\
	var->setFont( _font );								\
	l->addWidget( var);								\
	_frame = new QFrame(this); _frame->setFrameStyle(QFrame::VLine | QFrame::Raised);\
	_frame->setLineWidth(3); l->addWidget(_frame);

OpaleView::OpaleView( OpalePart* part, QWidget* parent, const char* name )
    : KoView( part, parent, name )
{
    setInstance( OpaleFactory::global() );
    setXMLFile( "opale.rc" );


	(void) new KAction( i18n("New &record"), "queue", QKeySequence("Ctrl+R" ), this, SLOT( new_record() ),
		actionCollection(), "new_record");
	(void) new KAction( i18n("&Repeat record"), "rebuild", 0, this, SLOT( repeat_record() ),
		actionCollection(), "repeat");
	(void) new KAction( i18n("&Delete record"), "editdelete", QKeySequence("Ctrl+D" ),
		this, SLOT( delete_record() ), actionCollection(), "delete_record" );


	// workaround for a bug in KListAction, who try to connect the slot to both activated() and activated(int)
	m_patternaction = new KListAction( i18n("New record &from pattern"), "editcopy", 0, 0, SLOT( new_record_from_pattern(int) ),
		actionCollection(), "new_record_from_pattern");

	connect( m_patternaction, SIGNAL(activated(int)), this, SLOT( new_record_from_pattern(int)) );
//	m_patternaction = new KListAction( i18n("New record &from pattern"), "editcopy", 0, this, SLOT( new_record_from_pattern(int) ),

	(void) new KAction( i18n("Edit file &options..."), "queue", 0, this, SLOT( editFileOptions() ),
		actionCollection(), "edit_file_options");

	(void) new KAction( i18n("New &pattern"), "queue", 0, this, SLOT( new_pattern() ),
		actionCollection(), "new_pattern");
	(void) new KAction( i18n("&Get graph"), "kchart", 0, this, SLOT( get_graph() ),
		actionCollection(), "get_graph");

	// get the data from part to display
	QGridLayout *grid = new QGridLayout(this, 2, 1);

	QBoxLayout * l = new QHBoxLayout( grid );

	QFrame *_frame;
	QFont _font("Utopia", 15);

	PUT_TOTAL( i18n("Balance :"), l_utt);
	PUT_TOTAL( i18n("Safe Balance :"), l_uttsafe);
	PUT_TOTAL( i18n("Checked Balance :"), l_checked);
//	PUT_TOTAL( i18n("Total :"), l_total);

	grid->addWidget(list=new KListView(this), 1,0);
	list->setAlternateBackground( QColor(190,190,255));
	list->addColumn(i18n("Date"));		// 0
	list->addColumn(i18n("Description"));
	list->addColumn(i18n("Amount"));		// 2
	list->addColumn(i18n("Checked"));
	list->addColumn(i18n("Type"));		// 4
	list->addColumn(i18n("Future?"));
	list->addColumn(i18n("Balance"));		// 6
	list->addColumn(i18n("Category"));
	list->setSorting(-1);
	list->setColumnAlignment( 2, AlignRight);
	list->setColumnAlignment( 3, AlignHCenter);
	list->setColumnAlignment( 5, AlignRight);
	list->setColumnAlignment( 6, AlignRight);
	list->setColumnAlignment( 7, AlignRight);

	current_dict = NULL; fillList();

	connect( list, SIGNAL(spacePressed(QListViewItem *)), this, SLOT(spacePressedRecord(QListViewItem *)));
	connect( list, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(editRecord(QListViewItem *)));
	connect( list, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(editRecord(QListViewItem *)));
	connect( part, SIGNAL(newPatterns(QStringList)), this, SLOT(newPatterns(QStringList)));
}


QString	OpaleView::fillLabel(amount_t amount, bool exact)
{
	QString	currency = opalepart()->currency();

	if (!exact) {
		int av = (amount>=0)?amount:-amount;
		if (av > 800000000) {
			amount /= 1000000;
			currency.prepend("M");
		} else if ( av > 800000) {
			amount /= 1000;
			currency.prepend("k");
		}
	}
	return QString("%1 %3").arg( QString().sprintf("%12.2f", amount/100.)).arg(currency);

}


void OpaleView::fillList(void)
{
	if (current_dict) delete current_dict;
	current_dict = opalepart()->fillList(&total, list);

//	l_total->setText( fillLabel(total.global) );
	l_utt->setText( fillLabel(total.utt) );
	l_uttsafe->setText( fillLabel(total.uttsafe) );
	l_checked->setText( fillLabel(total.checked, true) );
}


void OpaleView::updateReadWrite( bool /*readwrite*/ )
{
#ifdef __GNUC__
#warning TODO
#endif
}

void OpaleView::spacePressedRecord(QListViewItem *item)
{
	OpaleRecord *record = (*current_dict)[item];

	if (!record) {
		kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::spacePressedRecord(QListViewItem *item):unexpected item selected ?!" << endl;
		return;
	}

	if ( record->checked()) {
		// checked -> unchecked

		total.checked -= record->amount();
		if (!record->isFutur()) {
			total.uttsafe -= record->safeAmount();
		}
		l_uttsafe->setText( fillLabel(total.uttsafe) );
		l_checked->setText( fillLabel(total.checked, true) );

		item->setText(3, " ");
		record->unCheck();
	} else {	// unchecked -> checked

		total.checked += record->amount();
		record->check();
		if (!record->isFutur()) {
			total.uttsafe += record->safeAmount();
		}
		l_uttsafe->setText( fillLabel(total.uttsafe) );
		l_checked->setText( fillLabel(total.checked, true) );

		item->setText(3, "X");
	}
	opalepart()->touched();
}


void OpaleView::editFileOptions()
{
	
	if (opalepart()->editFileOptions(this))
		fillList();	// really not optimised..
}



void OpaleView::editRecord(QListViewItem *item)
{
	OpaleRecord *record = (*current_dict)[item];
	OpalePart *p = opalepart();
	int result;

	if (!record) {
		kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::editRecord(KListViewItem *item):unexpected item selected ?!" << endl;
		return;
	}

	inputRecordDialog dialog(this, i18n("Modify Record"), true);
	p->fillCombos(dialog.edit_how, dialog.edit_category);
	record->to_dialog(&dialog);

	result = dialog.exec();

	if (result) {
		OpaleRecord *newr = new OpaleRecord(&dialog,
					p->checkTypeCombos(dialog.edit_how),
					p->checkCategoryCombos(dialog.edit_category));
		if ( record->checked()) newr->check();
		p->deleteRecord(record);
		p->addRecord(newr);
		fillList();	// really not optimised..
	}
}


void OpaleView::newPatterns(QStringList l )
{
	m_patternaction->setItems(l);
}

void OpaleView::repeat_record(void)
{
	// presently cut is really a 'delete'
	kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::repeat_record() called" << endl;

	OpaleRecord *orig_record = (*current_dict)[list->currentItem()];

	/* gui */
	int result;

	inputRepeatDialog dialog(this, i18n("Repeat Record"), true);
	result = dialog.exec();

	if (!result) return; // nothing to do


	OpalePart *p = opalepart();
	int i;
	int n = dialog.spinBoxNb->value();
	int periodNb = dialog.spinBoxPeriod->value();
	int periodUnit = dialog.editPeriod->currentItem();

	for (i=0; i< n; i++) {
		OpaleRecord *r = new OpaleRecord(*orig_record);
		r->unCheck();
		r->addTime( (i+1)*periodNb, periodUnit);
		p->addRecord(r);
	}

	/* update gui */
	fillList();
}



void OpaleView::delete_record(void)
{
	// presently cut is really a 'delete'
	kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::cut() called" << endl;

	OpaleRecord *record = (*current_dict)[list->currentItem()];
	opalepart()->deleteRecord( record);
	delete list->currentItem();
	fillList();
}

void OpaleView::new_pattern(void)
{
	kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::new_pattern() called" << endl;

	OpalePattern pattern;
	int result;
	OpalePart *p = opalepart();

	inputPatternDialog dialog(this, i18n("Modify Pattern"), true);
	p->fillCombos(dialog.edit_how, dialog.edit_category);

	pattern.to_dialog(&dialog);

	result = dialog.exec();
	if (result) {
		p->addPattern(new OpalePattern(&dialog,
					p->checkTypeCombos(dialog.edit_how),
					p->checkCategoryCombos(dialog.edit_category)));

		fillList();	// really not optimised..
	}
}


void OpaleView::new_record_from_pattern(int i)
{
	kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::new_record_from_pattern() called" << endl;

	int result;
	OpalePart *p = opalepart();
	OpaleRecord *record = p->getRecordFromPattern(i);

	inputRecordDialog dialog(this, i18n("Modify Record"), true);
	p->fillCombos(dialog.edit_how, dialog.edit_category);

	record->to_dialog(&dialog);

	result = dialog.exec();

	delete record;

	if (result) {
		p->addRecord(new OpaleRecord(&dialog,
					p->checkTypeCombos(dialog.edit_how),
					p->checkCategoryCombos(dialog.edit_category)));

		fillList();	// really not optimised..
	}
}


void OpaleView::new_record(void)
{
	kdDebug(OPALE_DEBUG_STREAM) << "OpaleView::new_record() called" << endl;

	OpaleRecord record;
	int result;
	OpalePart *p = opalepart();

	inputRecordDialog dialog(this, i18n("Modify Record"), true);
	p->fillCombos(dialog.edit_how, dialog.edit_category);

	record.to_dialog(&dialog);

	result = dialog.exec();
	if (result) {
		p->addRecord(new OpaleRecord(&dialog,
					p->checkTypeCombos(dialog.edit_how),
					p->checkCategoryCombos(dialog.edit_category)));

		fillList();	// really not optimised..
	}
}

void OpaleView::get_graph(void)
{
	if (opalepart()->recordsCount() < 5) {
		KMessageBox::error(this, i18n("Sorry, you don't have enough data yet to display a chart"), i18n("Not enough data"));
		return;
	}
	OpaleGraph *og = new OpaleGraph(opalepart() , this, "opale_graph_mainwindow");
	connect( opalepart(), SIGNAL(dataUpdated(void)), og, SLOT(updateChart(void)));
	og->show();
	/* don't ask, i've spent too much time on this. maybe a bug in KDChart ? */
	/* yes, twice of them are needed.. dunno why */
	/* TODO : test with newer version of KDChart... */
	og->updateChart();
	og->updateChart();
}

